#  Copyright 2020 Google LLC
#  Copyright 2020 Amazon
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import configparser
import logging

from airflow.models import Variable
from libs.refresh_token import AirflowTokenRefresher, refresh_token
from osdu_api.model.acl import Acl
from osdu_api.model.legal import Legal
from osdu_api.model.legal_compliance import LegalCompliance
from osdu_api.model.record import Record
from osdu_api.model.record_ancestry import RecordAncestry
from osdu_api.storage.record_client import RecordClient

logger = logging.getLogger()

ACL_DICT = eval(Variable.get("acl"))
LEGAL_DICT = eval(Variable.get("legal"))

config = configparser.RawConfigParser()
config.read(Variable.get("dataload_config_path"))

DEFAULT_TENANT = config.get("DEFAULTS", "tenant")
DEFAULT_SOURCE = config.get("DEFAULTS", "authority")
DEFAULT_VERSION = config.get("DEFAULTS", "kind_version")


@refresh_token(AirflowTokenRefresher())
def create_update_record_request(headers, record_client, record):
    resp = record_client.create_update_records([record], headers.items())
    return resp


def create_records(**kwargs):
    # the only way to pass in values through the experimental api is through
    # the conf parameter
    data_conf = kwargs["dag_run"].conf

    logger.debug(kwargs)
    logger.debug(data_conf)

    acl = Acl(ACL_DICT["viewers"], ACL_DICT["owners"])
    legal = Legal(
        LEGAL_DICT["legaltags"],
        LEGAL_DICT["otherRelevantDataCountries"],
        LegalCompliance.compliant)
    auth = data_conf["authorization"]
    ancestry = RecordAncestry([])
    record_id = None

    # TODO: find out how to get kind through request not using Variables
    kind = Variable.get("record_kind")
    tenant, authority, file_type, version = kind.split(":")
    total_count = {file_type.capitalize(): 1}
    logger.info(f"The count of records to be ingested: {total_count}")

    meta = [{}]
    version = 0
    data = data_conf.get("data", {})
    record = Record(record_id, version, kind, acl, legal, data, ancestry, meta)

    headers = {
        "content-type": "application/json",
        "slb-data-partition-id": data_conf.get("partition-id", DEFAULT_SOURCE),
        "AppKey": data_conf.get("app-key", "")
    }

    record_client = RecordClient()
    record_client.data_partition_id = data_conf.get(
        "partition-id", DEFAULT_SOURCE)
    resp = create_update_record_request(headers, record_client, record)
    logger.info(f"Response: {resp.text}")
    kwargs["ti"].xcom_push(key="record_ids", value=resp.json()["recordIds"])
    return {"response_status": resp.status_code}
