#  Copyright 2020 Google LLC
#  Copyright 2020 EPAM Systems
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.


import copy
import json
import os
import sys

import libs.exceptions
import pytest

sys.path.append(f"{os.getenv('AIRFLOW_SRC_DIR')}/plugins")

from data import process_manifest_r3 as test_data
from operators import process_manifest_r3


@pytest.mark.parametrize(
    "input",
    [
        # pytest.param(test_data.CONF, id="Valid manifest"),
        # pytest.param(test_data.EMPTY_MANIFEST, marks=pytest.mark.xfail, id="Empty manifest"),
        pytest.param("data/workProduct/SeismicTraceData.json", id="Valid WorkProduct"),
        pytest.param("data/master/Wellbore.0.3.0.json", id="Valid master")
    ]
)
def test_create_manifest_records(input):
    with open(input) as f:
        conf = json.load(f)
    context = process_manifest_r3.Context.populate(conf)
    manifest_processor = process_manifest_r3.ManifestProcessor(
        storage_url="",
        dagrun_conf=conf,
        context=context
    )
    records = manifest_processor.create_manifest_records()
    records = json.dumps(records, indent=4)
    print(records)


@pytest.mark.parametrize(
    "schema,manifest",
    [
        pytest.param(test_data.TEST_SCHEMA, test_data.CONF, marks=pytest.mark.xfail,
                     id="Valid manifest"),
    ]
)
def test_schema_validator(schema, manifest):
    conf = copy.deepcopy(manifest)
    context = process_manifest_r3.Context.populate(conf)
    validator = process_manifest_r3.SchemaValidator(
        "",
        conf,
        context
    )
    validator._validate_schema(manifest["manifest"][0], schema)
