import os
import sys
from unittest.mock import MagicMock

import pytest

sys.path.append(f"{os.getenv('AIRFLOW_SRC_DIR')}/dags")

from libs.refresh_token import AirflowTokenRefresher
from libs.exceptions import SAFilePathError


def create_token_refresher() -> AirflowTokenRefresher:
    token_refresher = AirflowTokenRefresher()
    return token_refresher


@pytest.mark.parametrize(
    "access_token",
    [
        "test"
    ]
)
def test_access_token_cached(access_token: str):
    """
    Check if access token stored in Airflow Variables after refreshing it.
    """
    token_refresher = create_token_refresher()
    token_refresher.get_access_token_using_sa_file = MagicMock(return_value=access_token)
    token_refresher.refresh_token()
    assert token_refresher.airflow_variables.get("access_token") == access_token


@pytest.mark.parametrize(
    "access_token",
    [
        "test"
    ]
)
def test_authorization_header(access_token: str):
    token_refresher = create_token_refresher()
    token_refresher.get_access_token_using_sa_file = MagicMock(return_value=access_token)
    token_refresher.refresh_token()
    assert token_refresher.authorization_header.get("Authorization") == f"Bearer {access_token}"


def test_raise_sa_path_error_on_getting_absent_sa_file():
    token_refresher = create_token_refresher()
    with pytest.raises(SAFilePathError):
        token_refresher.get_sa_file_info()
